/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>

using namespace mrpt::utils;
using namespace mrpt::vision;
using namespace mrpt::gui;
using namespace mrpt::slam;
using namespace std;

// ------------------------------------------------------
//					Test
// ------------------------------------------------------
//void TestCapture_LandmarkMap()
//{
//	CStereoGrabber_Bumblebee	capture;
//	CObservationStereoImages		dualImage;
//	CObservationVisualLandmarks		obsLandmarks;
//	CTicTac							tictac;
//	bool							ok = true;
//
//	printf("Capturing...");
//	tictac.Tic();
//	ok = capture.getBothObservation( obsLandmarks, dualImage );
//	printf("...Done in %.03f ms, %u landmarks\n", 1000.0f*tictac.Tac(), (unsigned int)obsLandmarks.landmarks.size() );
//
//	obsLandmarks.landmarks.saveToTextFile(std::string("test.txt"));
//	dualImage.imageLeft.saveToFile("stereo_l.jpg");
//	dualImage.imageRight.saveToFile("stereo_r.jpg");
//
//}

// ------------------------------------------------------
//					TestCapture_Bumblebee
// ------------------------------------------------------
void TestCapture_Bumblebee()
{
	cout << "Press any key to stop capture to 'capture.rawlog'..." << endl;
	CFileGZOutputStream			fil("./capture.rawlog");
	CDisplayWindow				win("Capturing... [LEFT IMAGE]");
	CTicTac						tictac, tt2;
	int							cnt = 0;

	// Bumblebee options
	TCaptureOptions_bumblebee	options;
	options.frame_width		= 800;		// X-Resolution
	options.frame_height	= 600;		// Y-Resolution
	options.color			= false;	// Grayscale
	options.getRectified	= true;		// Rectified

	CStereoGrabber_Bumblebee	capture( 0, options );
	
	while( !mrpt::system::os::kbhit() )
	{
		if( (cnt++ % 20) == 0 )
		{
			if(cnt>0)
			{
				double t = tictac.Tac();
				double FPS = 20 / t;
				printf("\n Capturing stereo images at %f FPS\n", FPS);
			}
			tictac.Tic();
		}

		CObservationStereoImagesPtr stObs= CObservationStereoImages::Create();  // Memory will be freed by SF destructor in each loop.
		tt2.Tic();
		if( !capture.getStereoObservation( *stObs ) )
		{
			cerr << "Error retrieving images!" << endl;
			break;
		}
		printf("\n getStereoObservation %.5f ms\n", tt2.Tac()*1000);
		
		tt2.Tic();
		fil << stObs;
		printf("\n fill the rawlog %.5f ms\n", tt2.Tac()*1000);

		cout << "."; cout.flush();
		if( win.isOpen() )
			win.showImage( stObs->imageLeft );
	}

	cout << "Done" << endl;
} // end TestCapture_Bumblebee 


int main(int argc, char **argv)
{
	try
	{
		TestCapture_Bumblebee();

		return 0;
	} catch (std::exception &e)
	{
		std::cout << "MRPT exception caught: " << e.what() << std::endl;
		return -1;
	}
	catch (...)
	{
		printf("Another exception!!");
		return -1;
	}

}
