/* +---------------------------------------------------------------------------+
   |          The Mobile Robot Programming Toolkit (MRPT) C++ library          |
   |                                                                           |
   |                   http://mrpt.sourceforge.net/                            |
   |                                                                           |
   |   Copyright (C) 2005-2009  University of Malaga                           |
   |                                                                           |
   |    This software was written by the Machine Perception and Intelligent    |
   |      Robotics Lab, University of Malaga (Spain).                          |
   |    Contact: Jose-Luis Blanco  <jlblanco@ctima.uma.es>                     |
   |                                                                           |
   |  This file is part of the MRPT project.                                   |
   |                                                                           |
   |     MRPT is free software: you can redistribute it and/or modify          |
   |     it under the terms of the GNU General Public License as published by  |
   |     the Free Software Foundation, either version 3 of the License, or     |
   |     (at your option) any later version.                                   |
   |                                                                           |
   |   MRPT is distributed in the hope that it will be useful,                 |
   |     but WITHOUT ANY WARRANTY; without even the implied warranty of        |
   |     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         |
   |     GNU General Public License for more details.                          |
   |                                                                           |
   |     You should have received a copy of the GNU General Public License     |
   |     along with MRPT.  If not, see <http://www.gnu.org/licenses/>.         |
   |                                                                           |
   +---------------------------------------------------------------------------+ */

#include <mrpt/core.h>

using namespace mrpt::utils;
using namespace mrpt::gui;
using namespace mrpt::vision;
using namespace std;

#include <mrpt/examples_config.h>
string   myDataDir( MRPT_EXAMPLES_BASE_DIRECTORY + string("imageConvolutionFFT/") );

void TestRectifyImages()
{
    CImage im;
    CMatrix cam_matrix(3,3), dist_coeff(1,4);

    im.loadFromFile("/home/paco/Documents/Images/calib/L01.bmp");
    cam_matrix(0,0) = 938.8868; cam_matrix(0,1) = 0;            cam_matrix(0,2) = 367.8682;
    cam_matrix(1,0) = 0;        cam_matrix(1,1) = 938.8868;     cam_matrix(1,2) = 303.2578;
    cam_matrix(2,0) = 0;        cam_matrix(2,1) = 0;            cam_matrix(2,2) = 1;

    dist_coeff(0,0) = -0.3202480;
    dist_coeff(0,1) = -0.3470451;
    dist_coeff(0,2) = 0;
    dist_coeff(0,3) = 0;

    im.rectifyImageInPlace( cam_matrix, dist_coeff );

    im.saveToFile("/home/paco/Documents/Images/calib/L01_REC.bmp");
} // end TestRectifyImages

// ------------------------------------------------------
//				TestCapture
// ------------------------------------------------------
void TestExtractFeatures()
{
	CDisplayWindow		wind1;//,wind2,wind3,wind4;
	CFeatureExtraction	fExt;
	CFeatureList		featsHarris, featsKLT, featsSIFT_Hess, featsSIFT_Lowe, featsSIFT_Vedaldi, featsSURF;
	CImage			img;

	string the_img = myDataDir+string("test_image.jpg");

	if (!img.loadFromFile(the_img ))
	{
		cerr << "Cannot load " << the_img  << endl;
		return;
	}
	cout << "Loaded test image: " << the_img << endl;

	//TImageROI ROI( 200, 400, 0, img.getHeight()-1 );
	CTicTac	tictac;

	cout << "Extracting Harris features... [f_harris.txt]";
	tictac.Tic();
	fExt.options.featsType = featHarris;
	fExt.detectFeatures( img, featsHarris );
	cout << "Detected " << featsHarris.size() << " features in " << endl;
	cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	featsHarris.saveToTextFile("f_harris.txt");
	wind1.setWindowTitle("Harris detected features");
	wind1.showImageAndPoints( img, featsHarris );


	//cout << "Computing SIFT descriptors only ... [f_harris+sift.txt]";
	//tictac.Tic();
	//fExt.options.SIFTOptions.implementation = CFeatureExtraction::Hess;
	//fExt.computeSiftDescriptors( img, featsHarris );
	//cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	//featsHarris.saveToTextFile("f_harris+sift.txt");

	//cout << "Extracting KLT features... [f_klt.txt]";
	//tictac.Tic();
	//fExt.options.featsType = featKLT;
	//fExt.detectFeatures( img, featsKLT );
	//cout << "Detected " << featsKLT.size() << " features in " << endl;
	//cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	//featsKLT.saveToTextFile("f_klt.txt");
	//wind2.setWindowTitle("KLT detected features");
	//wind2.showImageAndPoints( img, featsKLT );

	//cout << "Extracting SIFT features... [f_sift_hess.txt]";
	//tictac.Tic();
	//fExt.options.featsType = featSIFT;
	//fExt.options.SIFTOptions.implementation = CFeatureExtraction::Hess;
	//fExt.detectFeatures( img, featsSIFT_Hess );
	//cout << "Detected " << featsSIFT_Hess.size() << " features in " << endl;
	//cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	//featsSIFT_Hess.saveToTextFile("f_sift_hess.txt");
	//wind3.setWindowTitle("SIFT Hess detected features");
	//wind3.showImageAndPoints( img, featsSIFT_Hess );

	//cout << "Extracting SURF features... [f_surf.txt]";
	//tictac.Tic();
	//fExt.options.featsType = featSURF;
	//fExt.detectFeatures( img, featsSURF );
	//cout << "Detected " << featsSURF.size() << " features in " << endl;
	//cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	//featsSURF.saveToTextFile("f_surf.txt");
	//wind4.setWindowTitle("SURF detected features");
	//wind4.showImageAndPoints( img, featsSURF );

	cout << "Computing spin images descriptors only ... [f_harris+spinimgs.txt]";
	tictac.Tic();
	fExt.options.SpinImagesOptions.radius = 13;
	fExt.options.SpinImagesOptions.hist_size_distance  = 10;
	fExt.options.SpinImagesOptions.hist_size_intensity = 10;

	fExt.computeDescriptors( img, featsHarris, descSpinImages );

	cout << format("  %.03fms",tictac.Tac()*1000) << endl;
	featsHarris.saveToTextFile("f_harris+spinimgs.txt");

	return; 
}

int main(int argc, char **argv)
{
	try
	{
		TestExtractFeatures();
		//TestRectifyImages();

		return 0;
	} catch (std::exception &e)
	{
		std::cout << "MRPT exception caught: " << e.what() << std::endl;
		return -1;
	}
	catch (...)
	{
		printf("Another exception!!");
		return -1;
	}

}
